@@ -3,7 +3,7 @@
  *
  *   Interactive command loop, C and Tcl callable.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1996 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1997 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXcmdloop.c,v 7.0 1996/06/16 05:30:09 markd Exp $
+ * $Id: tclXcmdloop.c,v 8.9 1997/07/04 20:23:43 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -59,9 +59,20 @@ static void
 AsyncCommandHandler _ANSI_ARGS_((ClientData clientData,
                                  int        mask));
 
+static int
+SyncSignalErrorHandler _ANSI_ARGS_((Tcl_Interp *interp,
+                                    ClientData  clientData,
+                                    int         background,
+                                    int         signalNum));
+
 static void
 AsyncCommandHandlerDelete _ANSI_ARGS_((ClientData clientData));
 
+static int 
+TclX_CommandloopObjCmd _ANSI_ARGS_((ClientData clientData, 
+                                    Tcl_Interp *interp,
+                                    int objc,
+                                    Tcl_Obj *CONST objv[]));
 
 /*-----------------------------------------------------------------------------
  * IsSetVarCmd --
@@ -78,8 +89,8 @@ static int
 IsSetVarCmd (command)
     char  *command;
 {
-    char  *nextPtr;
-    int    wordCnt;
+    char *nextPtr, *lastChar;
+    int wordCnt;
 
     if ((!STRNEQU (command, "set", 3)) || (!ISSPACE (command [3])))
         return FALSE;  /* Quick check */
@@ -89,8 +100,9 @@ IsSetVarCmd (command)
      */
     wordCnt = 0;
     nextPtr = command;
+    lastChar = command + strlen (command) - 1;
     while (*nextPtr != '\0') {
-        nextPtr = TclWordEnd (nextPtr, FALSE, NULL);
+        nextPtr = TclWordEnd (nextPtr, lastChar, FALSE, NULL);
         nextPtr++;  /* Character after the word */
         while ((*nextPtr != '\0') && (ISSPACE (*nextPtr)))
             nextPtr++;
@@ -106,7 +118,7 @@ IsSetVarCmd (command)
  *
  * Parameters:
  *   o interp (I) - A pointer to the interpreter.  Result of command should be
- *     in interp->result.
+ *     in interp result.
  *   o intResult (I) - The integer result returned by Tcl_Eval.
  *   o checkCmd (I) - If not NULL and the command was sucessful, check to
  *     set if this is a "set" command setting a variable.  If so, don't echo
@@ -120,6 +132,7 @@ TclX_PrintResult (interp, intResult, checkCmd)
     char       *checkCmd;
 {
     Tcl_Channel stdoutChan,  stderrChan;
+    char *resultStr;
 
     /*
      * If the command was supplied and it was a successful set of a variable,
@@ -134,10 +147,11 @@ TclX_PrintResult (interp, intResult, checkCmd)
     if (intResult == TCL_OK) {
         if (stdoutChan == NULL)
             return;
-        if (interp->result [0] != '\0') {
+        resultStr = Tcl_GetStringFromObj (Tcl_GetObjResult (interp), NULL);
+        if (resultStr [0] != '\0') {
             if (stderrChan != NULL)
                 Tcl_Flush (stderrChan);
-            TclX_WriteStr (stdoutChan, interp->result);
+            TclX_WriteStr (stdoutChan, resultStr);
             TclX_WriteNL (stdoutChan);
             Tcl_Flush (stdoutChan);
         }
@@ -154,8 +168,9 @@ TclX_PrintResult (interp, intResult, checkCmd)
         } else {
             sprintf (msg, "Bad return code (%d): ", intResult);
         }
+        resultStr = Tcl_GetStringFromObj (Tcl_GetObjResult (interp), NULL);
         TclX_WriteStr (stderrChan, msg);
-        TclX_WriteStr (stderrChan, interp->result);
+        TclX_WriteStr (stderrChan, resultStr);
         TclX_WriteNL (stderrChan);
         Tcl_Flush (stderrChan);
     }
@@ -186,7 +201,7 @@ OutputPrompt (interp, topLevel, prompt1, prompt2)
     char       *prompt1;
     char       *prompt2;
 {
-    char *promptHook;
+    char *promptHook, *resultStr;
     int result, useResult, promptDone = FALSE;
     Tcl_Channel stdoutChan, stderrChan;
 
@@ -227,15 +242,16 @@ OutputPrompt (interp, topLevel, prompt1, prompt2)
 
     if (promptHook != NULL) {
         result = Tcl_Eval (interp, promptHook);
+        resultStr = Tcl_GetStringFromObj (Tcl_GetObjResult (interp), NULL);
         if (result == TCL_ERROR) {
             if (stderrChan != NULL) {
                 TclX_WriteStr (stderrChan, "Error in prompt hook: ");
-                TclX_WriteStr (stderrChan, interp->result);
+                TclX_WriteStr (stderrChan, resultStr);
                 TclX_WriteNL (stderrChan);
             }
         } else {
             if (useResult && (stdoutChan != NULL))
-                TclX_WriteStr (stdoutChan, interp->result);
+                TclX_WriteStr (stdoutChan, resultStr);
             promptDone = TRUE;
         }
     } 
@@ -311,7 +327,7 @@ AsyncCommandHandler (clientData, mask)
 {
     asyncLoopData_t *dataPtr = (asyncLoopData_t *) clientData;
     int code;
-    char *cmd;
+    char *cmd, *resultStr;
 
     /*
      * Make sure that we are the current signal error handler.  This
@@ -363,7 +379,10 @@ AsyncCommandHandler (clientData, mask)
     code = Tcl_RecordAndEval (dataPtr->interp, cmd, TCL_EVAL_GLOBAL);
     Tcl_CreateChannelHandler (dataPtr->channel, TCL_READABLE,
                               AsyncCommandHandler, clientData);
-    if (dataPtr->interp->result [0] != '\0') {
+
+    resultStr = Tcl_GetStringFromObj (Tcl_GetObjResult (dataPtr->interp),
+                                      NULL);
+    if (resultStr [0] != '\0') {
         if (dataPtr->options & TCLX_CMDL_INTERACTIVE) {
             TclX_PrintResult (dataPtr->interp, code, cmd);
         }
@@ -424,7 +443,7 @@ AsyncCommandHandlerDelete (clientData)
         ckfree (dataPtr->prompt1);
     if (dataPtr->prompt2 != NULL)
         ckfree (dataPtr->prompt2);
-    ckfree (dataPtr);
+    ckfree ((char *) dataPtr);
 }
 
 /*-----------------------------------------------------------------------------
@@ -561,7 +580,8 @@ TclX_CommandLoop (interp, options, endCommand, prompt1, prompt2)
     char       *prompt2;
 {
     Tcl_DString command;
-    int result, partial = FALSE, gotSigIntError = FALSE;
+    int result, partial = FALSE, gotSigIntError = FALSE,
+      gotInterrupted = FALSE;
     Tcl_Channel stdinChan, stdoutChan;
 
     Tcl_DStringInit (&command);
@@ -587,14 +607,12 @@ TclX_CommandLoop (interp, options, endCommand, prompt1, prompt2)
          * were through here, event if its already been processed.
          */
         if (gotSigIntError) {
-            gotSigIntError = FALSE;
             Tcl_DStringFree (&command);
             partial = FALSE;
             stdoutChan = Tcl_GetStdChannel (TCL_STDOUT);
             if (stdoutChan != NULL)
                 TclX_WriteNL (stdoutChan);
         }
-        gotSigIntError = FALSE;
 
         /*
          * Output a prompt and input a command.
@@ -603,19 +621,31 @@ TclX_CommandLoop (interp, options, endCommand, prompt1, prompt2)
         if (stdinChan == NULL)
             goto endOfFile;
 
-        if (options & TCLX_CMDL_INTERACTIVE) {
+        /*
+         * Only ouput prompt if we didn't get interrupted or if the
+         * interruption was SIGINT
+         */
+        if ((options & TCLX_CMDL_INTERACTIVE) &&
+            (!gotInterrupted || gotSigIntError)) {
             OutputPrompt (interp, !partial, prompt1, prompt2);
         }
 
+        /*
+         * Reset these flags for the next round
+         */
+        gotSigIntError = FALSE;
+        gotInterrupted = FALSE; 
+
         result = Tcl_Gets (stdinChan, &command);
         if (result < 0) {
             if (Tcl_Eof (stdinChan) || Tcl_InputBlocked (stdinChan))
                 goto endOfFile;
             if (Tcl_GetErrno () == EINTR) {
+                gotInterrupted = TRUE; 
                 continue;  /* Process signals above */
             }
-            Tcl_AppendResult (interp, "command input error on stdin: ",
-                              Tcl_PosixError (interp), (char *) NULL);
+            TclX_AppendObjResult (interp, "command input error on stdin: ",
+                                  Tcl_PosixError (interp), (char *) NULL);
             return TCL_ERROR;
         }
 
@@ -654,7 +684,7 @@ TclX_CommandLoop (interp, options, endCommand, prompt1, prompt2)
 }
 
 /*-----------------------------------------------------------------------------
- * Tcl_CommandloopCmd --
+ * Tcl_CommandloopObjCmd --
  *    Implements the commandloop command:
  *       commandloop -async -interactive on|off|tty -prompt1 cmd
  *                   -prompt2 cmd -endcommand cmd
@@ -662,49 +692,53 @@ TclX_CommandLoop (interp, options, endCommand, prompt1, prompt2)
  *   Standard TCL results.
  *-----------------------------------------------------------------------------
  */
-int
-Tcl_CommandloopCmd (clientData, interp, argc, argv)
+static int
+TclX_CommandloopObjCmd (clientData, interp, objc, objv)
     ClientData  clientData;
     Tcl_Interp *interp;
-    int         argc;
-    char      **argv;
+    int         objc;
+    Tcl_Obj    *CONST objv[];
 {
     int options = 0, async = FALSE, argIdx, interactive;
-    char *endCommand = NULL;
+    char *argStr,  *endCommand = NULL;
     char *prompt1 = NULL, *prompt2 = NULL;
 
     interactive = isatty (0);
-    for (argIdx = 1; (argIdx < argc) && (argv [argIdx][0] == '-'); argIdx++) {
-        if (STREQU (argv [argIdx], "-async")) {
+    for (argIdx = 1; argIdx < objc; argIdx++) {
+        argStr = Tcl_GetStringFromObj (objv [argIdx], NULL);
+        if (argStr [0] != '-')
+            break;
+        if (STREQU (argStr, "-async")) {
             async = TRUE;
-        } else if (STREQU (argv [argIdx], "-prompt1")) {
-            if (argIdx == argc - 1)
+        } else if (STREQU (argStr, "-prompt1")) {
+            if (argIdx == objc - 1)
                 goto argRequired;
-            prompt1 = argv [++argIdx];
-        } else if (STREQU (argv [argIdx], "-prompt2")) {
-            if (argIdx == argc - 1)
+            prompt1 = Tcl_GetStringFromObj (objv [++argIdx], NULL);;
+        } else if (STREQU (argStr, "-prompt2")) {
+            if (argIdx == objc - 1)
                 goto argRequired;
-            prompt2 = argv [++argIdx];
-        } else if (STREQU (argv [argIdx], "-interactive")) {
-            if (argIdx == argc - 1)
+            prompt2 = Tcl_GetStringFromObj (objv [++argIdx], NULL);
+        } else if (STREQU (argStr, "-interactive")) {
+            if (argIdx == objc - 1)
                 goto argRequired;
             argIdx++;
-            if (STREQU (argv [argIdx], "tty")) {
+            argStr = Tcl_GetStringFromObj (objv [argIdx], NULL);
+            if (STREQU (argStr, "tty")) {
                 interactive = TRUE;
             } else {
-                if (Tcl_GetBoolean (interp, argv [argIdx],
-                                    &interactive) != TCL_OK)
+                if (Tcl_GetBooleanFromObj (interp, objv [argIdx],
+                                           &interactive) != TCL_OK)
                     return TCL_ERROR;
             }
-        } else if (STREQU (argv [argIdx], "-endcommand")) {
-            if (argIdx == argc - 1)
+        } else if (STREQU (argStr, "-endcommand")) {
+            if (argIdx == objc - 1)
                 goto argRequired;
-            endCommand = argv [++argIdx];
+            endCommand = Tcl_GetStringFromObj (objv [++argIdx], NULL);
         } else {
             goto unknownOption;
         }
     }
-    if (argIdx != argc)
+    if (argIdx != objc)
         goto wrongArgs;
 
     if (interactive)
@@ -726,25 +760,41 @@ Tcl_CommandloopCmd (clientData, interp, argc, argv)
 
 
     /*
-     * Argument error message generation.  argv [argIdx] should contain the
+     * Argument error message generation.  argStr should contain the
      * option being processed.
      */
   argRequired:
-    Tcl_AppendResult (interp, "argument required for ", argv [argIdx],
-                      " option", (char *) NULL);
+    TclX_AppendObjResult (interp, "argument required for ", argStr,
+                          " option", (char *) NULL);
     return TCL_ERROR;
 
   unknownOption:
-    Tcl_AppendResult (interp, "unknown option \"", argv [argIdx],
-                      "\", expected one of \"-async\", ",
-                      "\"-interactive\", \"-prompt1\", \"-prompt2\", ",
-                      " or \"-endcommand\"", (char *) NULL);
+    TclX_AppendObjResult (interp, "unknown option \"", argStr,
+                          "\", expected one of \"-async\", ",
+                          "\"-interactive\", \"-prompt1\", \"-prompt2\", ",
+                          " or \"-endcommand\"", (char *) NULL);
     return TCL_ERROR;
     
   wrongArgs:
-    Tcl_AppendResult (interp, tclXWrongArgs, argv [0], " ?-async? ",
-                      "?-interactive on|off|tty? ?-prompt1 cmd? ",
-                      "?-prompt2 cmd? ?-endcommand cmd?",
-                      (char *) NULL);
+    TclX_WrongArgs (interp, objv [0],
+                    "?-async? ?-interactive on|off|tty? ?-prompt1 cmd? ?-prompt2 cmd? ?-endcommand cmd?");
     return TCL_ERROR;
 }
+
+/*-----------------------------------------------------------------------------
+ * TclX_CmdloopInit --
+ *     Initialize the coommandloop command.
+ *-----------------------------------------------------------------------------
+ */
+void
+TclX_CmdloopInit (interp)
+    Tcl_Interp *interp;
+{
+    Tcl_CreateObjCommand (interp,
+                          "commandloop",
+                          TclX_CommandloopObjCmd, 
+                          (ClientData) NULL,
+                          (Tcl_CmdDeleteProc*) NULL);
+    
+}
+
