@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXutil.c,v 8.20 1997/08/16 16:25:58 markd Exp $
+ * $Id: tclXutil.c,v 8.21.2.1 1998/04/26 20:03:54 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -1064,7 +1064,8 @@ TclX_WrongArgs (interp, commandNameObj, string)
  * TclX_AppendObjResult --
  *
  *   Append a variable number of strings onto the object result already
- * present for an interpreter.
+ * present for an interpreter.  If the object is shared, the current contents
+ * are discarded.
  *
  * Parameters:
  *   o interp - Interpreter to set the result in.
@@ -1082,6 +1083,11 @@ TclX_AppendObjResult TCL_VARARGS_DEF (Tcl_Interp *, arg1)
     interp = TCL_VARARGS_START (Tcl_Interp *, arg1, argList);
     resultPtr = Tcl_GetObjResult (interp);
 
+    if (Tcl_IsShared(resultPtr)) {
+        resultPtr = Tcl_NewStringObj(0, NULL);
+        Tcl_SetObjResult(interp, resultPtr);
+    }
+
     TCL_VARARGS_START(Tcl_Interp *,arg1,argList);
     while (1) {
         string = va_arg(argList, char *);
@@ -1152,7 +1158,11 @@ Tcl_Obj *
 TclX_SaveResultErrorInfo (interp)
     Tcl_Interp  *interp;
 {
-    Tcl_Obj *saveObjv [3];
+    Tcl_Obj *saveObjv [4];
+    Tcl_Obj *listObj;
+
+    long flags = ((Interp *)interp)->flags &
+	(ERR_ALREADY_LOGGED | ERR_IN_PROGRESS | ERROR_CODE_SET);
 
     saveObjv [0] = Tcl_DuplicateObj (Tcl_GetObjResult (interp));
     
@@ -1168,7 +1178,11 @@ TclX_SaveResultErrorInfo (interp)
         saveObjv [2] = Tcl_NewObj ();
     }
 
-    return Tcl_NewListObj (3, saveObjv);
+    saveObjv [3] = Tcl_NewLongObj(flags);
+
+    Tcl_IncrRefCount(listObj = Tcl_NewListObj (4, saveObjv));
+
+    return listObj;
 }
 
 
@@ -1190,27 +1204,27 @@ TclX_RestoreResultErrorInfo (interp, saveObjPtr)
 {
     Tcl_Obj **saveObjv;
     int saveObjc;
-    char *errorInfo;
-    int errorInfoLen;
+    long flags;
 
     if ((Tcl_ListObjGetElements (NULL, saveObjPtr, &saveObjc,
                                  &saveObjv) != TCL_OK) ||
-        (saveObjc != 3)) {
+        (saveObjc != 4) ||
+        (Tcl_GetLongFromObj (NULL, saveObjv[3], &flags) != TCL_OK)) {
+	/*
+	 * This should never happen
+	 */
         panic ("invalid TclX result save object");
     }
 
-    /*
-     * Need to clear flags and result so that errorInfo/errorCode are set
-     * correctly.
-     */
-    Tcl_ResetResult (interp);
+    TclX_ObjSetVar2S (interp, ERRORCODE, NULL, saveObjv[2],
+		      TCL_GLOBAL_ONLY);
 
-    Tcl_SetObjErrorCode (interp, saveObjv [2]);
+    TclX_ObjSetVar2S (interp, ERRORINFO, NULL, saveObjv[1],
+		      TCL_GLOBAL_ONLY);
 
-    errorInfo = Tcl_GetStringFromObj (saveObjv [1], &errorInfoLen);
-    Tcl_AddObjErrorInfo (interp, errorInfo, errorInfoLen);
+    Tcl_SetObjResult (interp, saveObjv[0]);
 
-    Tcl_SetObjResult (interp, saveObjv [0]);
+    ((Interp *)interp)->flags |= flags;
 
     Tcl_DecrRefCount (saveObjPtr);
 }
